@@ -15,4 +15,10 @@ class DetectionMode(Mode):
         self.name = 'Detection Mode'
 
     def mousePressEvent(self, event: QMouseEvent) -> None:
-        self.viewer.detect_mesh_intersection(event.pos().x(), event.pos().y(), 1.0)
+        super().mousePressEvent(event)
+
+        x, y, z = [event.pos().x(), event.pos().y(), 1]
+        origin = self.viewer.origin_from_click(x, y, z)
+        ray = self.viewer.ray_from_click(x, y, z)
+
+        self.viewer.intersect_elements(origin, ray)
