@@ -18,10 +18,15 @@ class MeshGL(GLDrawable):
 
         self._highlighted = kwargs.pop('highlight', False)
         self._wireframed = kwargs.pop('wireframe', False)
+        self._cross_sectionable = kwargs.pop('cross_section', False)
 
     """
     Properties
     """
+    @property
+    def is_standard(self) -> bool:
+        return not (self.is_highlighted or self.is_wireframed)
+
     @property
     def is_highlighted(self) -> bool:
         return self._highlighted
@@ -32,7 +37,11 @@ class MeshGL(GLDrawable):
 
     @property
     def is_turbo_ready(self) -> bool:
-        return self.is_boostable and not self.is_highlighted and not self.is_wireframed
+        return self.is_boostable and self.is_standard
+
+    @property
+    def is_cross_sectionable(self) -> bool:
+        return self._cross_sectionable
 
     @is_highlighted.setter
     def is_highlighted(self, status: bool) -> None:
@@ -44,6 +53,11 @@ class MeshGL(GLDrawable):
         self._wireframed = status
         self.notify()
 
+    @is_cross_sectionable.setter
+    def is_cross_sectionable(self, status: bool) -> None:
+        self._cross_sectionable = status
+        self.notify()
+
     """
     Quick MeshGL API
     """
