@@ -39,20 +39,11 @@ class ActionCollection:
         self.action_setup_colors = QAction('Setup c&olors', parent)
         self.action_setup_colors.setIcon(IconCollection.get('picture.svg'))
 
-        self.action_export_mesh = QAction('&Export mesh', parent)
-        self.action_export_mesh.setIcon(IconCollection.get('export.svg'))
+        self.action_setup_multiple = QAction('Setup &multiple', parent)
+        self.action_setup_multiple.setIcon(IconCollection.get('picture.svg'))
 
-        self.action_export_blocks = QAction('Export &blocks', parent)
-        self.action_export_blocks.setIcon(IconCollection.get('export.svg'))
-
-        self.action_export_points = QAction('Export &points', parent)
-        self.action_export_points.setIcon(IconCollection.get('export.svg'))
-
-        self.action_export_lines = QAction('Export &lines', parent)
-        self.action_export_lines.setIcon(IconCollection.get('export.svg'))
-
-        self.action_export_tubes = QAction('Export &tubes', parent)
-        self.action_export_tubes.setIcon(IconCollection.get('export.svg'))
+        self.action_export_element = QAction('&Export', parent)
+        self.action_export_element.setIcon(IconCollection.get('export.svg'))
 
         # MainWindow Actions
         self.action_load_mesh = QAction('Load &mesh', parent)
@@ -139,7 +130,7 @@ class ActionCollection:
         self.action_measurement_mode.setIcon(IconCollection.get('ruler.svg'))
         self.action_measurement_mode.setShortcut('Ctrl+M')
 
-        self.action_detection_mode = QAction('&Detect meshes', parent)
+        self.action_detection_mode = QAction('&Detect meshes/lines', parent)
         self.action_detection_mode.setIcon(IconCollection.get('cursor.svg'))
         self.action_detection_mode.setShortcut('Ctrl+D')
 
