@@ -40,33 +40,52 @@ class MainWindow(QMainWindow):
         self.statusBar.showMessage('Ready')
 
         # Attributes
-        self.settings = QSettings('BlastSight', application='blastsight', parent=self)
         self.filters_dict = {
-            'mesh': 'Mesh Files (*.dxf *.off *.h5m);;'
-                    'DXF Files (*.dxf);;'
-                    'OFF Files (*.off);;'
-                    'H5M Files (*.h5m);;'
-                    'All Files (*.*)',
-            'block': 'Block Files (*.csv *.h5p *.out);;'
-                     'CSV Files (*.csv);;'
-                     'H5P Files (*.h5p);;'
-                     'GSLib Files (*.out);;'
-                     'All Files (*.*)',
-            'point': 'Point Files (*.csv *.h5p *.out);;'
-                     'CSV Files (*.csv);;'
-                     'H5P Files (*.h5p);;'
-                     'GSLib Files (*.out);;'
-                     'All Files (*.*)',
-            'line': 'Line Files (*.csv *.dxf);;'
-                    'CSV Files (*.csv);;'
-                    'DXF Files (*.csv);;'
-                    'All Files (*.*)',
-            'tube': 'Tube Files (*.csv *.dxf);;'
-                    'CSV Files (*.csv);;'
-                    'DXF Files (*.csv);;'
-                    'All Files (*.*)',
+            'mesh': {
+                'load': 'Mesh Files (*.dxf *.off *.h5m);;'
+                        'DXF Files (*.dxf);;'
+                        'OFF Files (*.off);;'
+                        'H5M Files (*.h5m);;'
+                        'All Files (*.*)',
+                'export': 'BlastSight Mesh (*.h5m);;'
+                          'OFF File (*.off);;'
+            },
+            'block': {
+                'load': 'Block Files (*.csv *.h5p *.out);;'
+                        'CSV Files (*.csv);;'
+                        'H5P Files (*.h5p);;'
+                        'GSLib Files (*.out);;'
+                        'All Files (*.*)',
+                'export': 'BlastSight Blocks (*.h5p);;'
+                          'CSV File (*.csv);;'
+            },
+            'point': {
+                'load': 'Point Files (*.csv *.h5p *.out);;'
+                        'CSV Files (*.csv);;'
+                        'H5P Files (*.h5p);;'
+                        'GSLib Files (*.out);;'
+                        'All Files (*.*)',
+                'export': 'BlastSight Points (*.h5p);;'
+                          'CSV File (*.csv);;'
+            },
+            'line': {
+                'load': 'Line Files (*.csv *.dxf);;'
+                        'CSV Files (*.csv);;'
+                        'DXF Files (*.dxf);;'
+                        'All Files (*.*)',
+                'export': 'CSV File (*.csv);;'
+            },
+            'tube': {
+                'load': 'Line Files (*.csv *.dxf);;'
+                        'CSV Files (*.csv);;'
+                        'DXF Files (*.dxf);;'
+                        'All Files (*.*)',
+                'export': 'CSV File (*.csv);;'
+            }
         }
 
+        self.settings = QSettings('BlastSight', application='blastsight', parent=self)
+
         # Progress bar (hidden by default)
         self.progress_bar = QProgressBar(self.statusBar)
         self.progress_bar.setValue(0)
@@ -103,6 +122,7 @@ class MainWindow(QMainWindow):
     def generate_menubar(self) -> None:
         actions = self.toolbar.action_collection
 
+        # File
         self.menu_File.addAction(actions.action_load_mesh)
         self.menu_File.addAction(actions.action_load_blocks)
         self.menu_File.addAction(actions.action_load_points)
@@ -117,22 +137,19 @@ class MainWindow(QMainWindow):
         self.menu_File.addSeparator()
         self.menu_File.addAction(actions.action_quit)
 
+        # View
         self.menu_View.addAction(actions.action_camera_properties)
         self.menu_View.addAction(actions.action_plan_view)
         self.menu_View.addAction(actions.action_north_view)
         self.menu_View.addAction(actions.action_east_view)
         self.menu_View.addAction(actions.action_fit_to_screen)
         self.menu_View.addSeparator()
-        self.menu_View.addAction(actions.action_autofit)
-        self.menu_View.addAction(actions.action_animate)
-        self.menu_View.addAction(actions.action_autorotate)
-        self.menu_View.addAction(actions.action_turbo_rendering)
-        self.menu_View.addSeparator()
         self.menu_View.addAction(actions.action_perspective_projection)
         self.menu_View.addAction(actions.action_orthographic_projection)
         self.menu_View.addSeparator()
         self.menu_View.addAction(actions.action_take_screenshot)
 
+        # Tools
         self.menu_Tools.addAction(actions.action_slice_meshes)
         self.menu_Tools.addAction(actions.action_slice_blocks)
         self.menu_Tools.addAction(actions.action_detection_mode)
@@ -141,6 +158,13 @@ class MainWindow(QMainWindow):
         self.menu_Tools.addAction(actions.action_normal_mode)
         self.menu_Tools.addAction(actions.action_cross_section)
 
+        # Settings
+        self.menu_Settings.addAction(actions.action_autofit)
+        self.menu_Settings.addAction(actions.action_animate)
+        self.menu_Settings.addAction(actions.action_autorotate)
+        self.menu_Settings.addAction(actions.action_turbo_rendering)
+
+        # Help
         self.menu_Help.addAction(actions.action_help)
         self.menu_Help.addAction(actions.action_about)
 
@@ -189,7 +213,7 @@ class MainWindow(QMainWindow):
         # Viewer signals
         self.viewer.signal_fps_updated.connect(self.print_fps)
         self.viewer.signal_mode_updated.connect(self.slot_mode_updated)
-        self.viewer.signal_mesh_clicked.connect(self.slot_mesh_clicked)
+        self.viewer.signal_elements_detected.connect(self.slot_elements_detected)
         self.viewer.signal_mesh_distances.connect(self.slot_mesh_distances)
 
         self.viewer.signal_load_success.connect(self.slot_element_load_success)
@@ -202,11 +226,7 @@ class MainWindow(QMainWindow):
         self.viewer.signal_process_finished.connect(self.slot_process_finished)
 
         # TreeWidget actions
-        self.treeWidget.signal_export_mesh.connect(self.dialog_export_mesh)
-        self.treeWidget.signal_export_blocks.connect(self.dialog_export_blocks)
-        self.treeWidget.signal_export_points.connect(self.dialog_export_points)
-        self.treeWidget.signal_export_lines.connect(self.dialog_export_lines)
-        self.treeWidget.signal_export_tubes.connect(self.dialog_export_tubes)
+        self.treeWidget.signal_export_element.connect(self._dialog_export_element)
 
     @property
     def last_dir(self) -> str:
@@ -242,9 +262,9 @@ class MainWindow(QMainWindow):
     def slot_mesh_distances(self, distance_dict: dict) -> None:
         self.statusBar.showMessage(f'Distance: {distance_dict.get("distance")}')
 
-    def slot_mesh_clicked(self, mesh_attributes: list) -> None:
-        id_list = [attr.get('id', -1) for attr in mesh_attributes]
-        self.statusBar.showMessage(f'Detected meshes: {id_list}')
+    def slot_elements_detected(self, attributes: list) -> None:
+        id_list = sorted(map(lambda attr: attr.get('id', -1), attributes))
+        self.statusBar.showMessage(f'Detected elements: {id_list}')
 
     """
     Slots for cross-sections
@@ -309,29 +329,34 @@ class MainWindow(QMainWindow):
         self.viewer.update_all()
 
     def add_mesh_slices(self, slice_list: list) -> None:
-        for sliced_meshes in slice_list:
-            slices = sliced_meshes.get('vertices')
-            mesh_id = sliced_meshes.get('element_id')
+        def add_slice(description: dict) -> None:
+            slices = description.get('vertices')
+            mesh_id = description.get('element_id')
             mesh = self.viewer.get_drawable(mesh_id)
 
-            if 'SLICE' in str(mesh.name):
-                continue
-
-            for i, vert_slice in enumerate(slices):
-                self.viewer.lines(vertices=vert_slice,
+            def add_subslice(subslice, num: int) -> None:
+                self.viewer.lines(vertices=subslice,
                                   color=mesh.color,
-                                  name=f'MESHSLICE_{i}_{mesh.name}',
+                                  name=f'MESHSLICE_{num}_{mesh.name}',
                                   extension='csv',
                                   loop=True)
 
+            # Execute add_subslice for all subslices of the slice
+            for i, ssl in enumerate(slices):
+                add_subslice(ssl, i)
+
+        # Execute add_slice over all slice descriptions
+        for sl in slice_list:
+            add_slice(sl)
+
     def add_block_slices(self, slice_list: list) -> None:
-        for sliced_blocks in slice_list:
-            indices = sliced_blocks.get('indices')
-            block_id = sliced_blocks.get('element_id')
+        def add_slice(description: dict) -> None:
+            indices = description.get('indices')
+            block_id = description.get('element_id')
             block = self.viewer.get_drawable(block_id)
 
             if 'SLICE' in str(block.name):
-                continue
+                return
 
             self.viewer.blocks(vertices=block.vertices[indices],
                                values=block.values[indices],
@@ -343,8 +368,12 @@ class MainWindow(QMainWindow):
                                name=f'BLOCKSLICE_{block.name}',
                                extension='csv')
 
+        # Execute add_slice over all slice descriptions
+        for sl in slice_list:
+            add_slice(sl)
+
     """
-    Common functionality for loading/exporting
+    Common functionality for loading
     """
     @staticmethod
     def _thread_runner(method: callable, *args, **kwargs) -> None:
@@ -355,7 +384,7 @@ class MainWindow(QMainWindow):
         (paths, selected_filter) = QFileDialog.getOpenFileNames(
             parent=self,
             directory=self.last_dir,
-            filter=self.filters_dict.get(hint))
+            filter=self.filters_dict.get(hint).get('load'))
 
         path_list = sorted([p for p in paths if p != ''])
         if len(path_list) > 0:
@@ -375,19 +404,6 @@ class MainWindow(QMainWindow):
             self._thread_runner(loader, path, *args, **kwargs)
             self.last_dir = path
 
-    def _dialog_export_element(self, _id: int, filters: str, method: classmethod) -> None:
-        proposed_path = QDir(self.last_dir).filePath(self.viewer.get_drawable(_id).element.name)
-
-        (path, selected_filter) = QFileDialog.getSaveFileName(
-            parent=self,
-            directory=proposed_path,
-            filter=filters)
-
-        # Execute method
-        if bool(path):
-            self.statusBar.showMessage('Exporting...')
-            self._thread_runner(method, path, _id)
-
     """
     Slots for progress updates
     """
@@ -446,32 +462,24 @@ class MainWindow(QMainWindow):
         self._dialog_load_folder(loader=self.viewer.load_tubes_folder)
 
     """
-    Slots for exporting files
+    Slots for exporting elements
     """
-    def dialog_export_mesh(self, _id: int) -> None:
-        self._dialog_export_element(_id=_id,
-                                    filters='BlastSight mesh (*.h5m);;',
-                                    method=self.viewer.export_mesh)
+    def _dialog_export_element(self, _id: int) -> None:
+        element = self.viewer.get_drawable(_id).element
+        proposed_path = QDir(self.last_dir).filePath(element.name)
 
-    def dialog_export_blocks(self, _id: int) -> None:
-        self._dialog_export_element(_id=_id,
-                                    filters='BlastSight blocks (*.h5p);;',
-                                    method=self.viewer.export_blocks)
+        # TODO Check hints by element type
+        filters = 'BlastSight FIXME (*.h5m);;'
 
-    def dialog_export_points(self, _id: int) -> None:
-        self._dialog_export_element(_id=_id,
-                                    filters='BlastSight points (*.h5p);;',
-                                    method=self.viewer.export_points)
+        (path, selected_filter) = QFileDialog.getSaveFileName(
+            parent=self,
+            directory=proposed_path,
+            filter=filters)
 
-    def dialog_export_lines(self, _id: int) -> None:
-        self._dialog_export_element(_id=_id,
-                                    filters='BlastSight lines (*.csv);;',
-                                    method=self.viewer.export_lines)
-
-    def dialog_export_tubes(self, _id: int) -> None:
-        self._dialog_export_element(_id=_id,
-                                    filters='BlastSight tubes (*.csv);;',
-                                    method=self.viewer.export_tubes)
+        # Execute method
+        if bool(path):
+            self.statusBar.showMessage('Exporting...')
+            self._thread_runner(self.viewer.export_element, path, _id)
 
     """
     Slots for modifying interaction modes
