@@ -6,7 +6,6 @@
 #  See LICENSE for more info.
 
 import numpy as np
-import warnings
 
 from OpenGL.GL import *
 
@@ -43,6 +42,7 @@ from ..controller.normalmode import NormalMode
 from ..controller.slicemode import SliceMode
 from ..controller.measurementmode import MeasurementMode
 
+from ..model import utils
 from ..model.model import Model
 
 
@@ -54,9 +54,9 @@ class IntegrableViewer(QOpenGLWidget):
     signal_export_success = Signal(int)
     signal_export_failure = Signal()
 
-    signal_mesh_clicked = Signal(object)
     signal_mesh_distances = Signal(object)
     signal_slice_description = Signal(object)
+    signal_elements_detected = Signal(object)
 
     signal_process_updated = Signal(int)
     signal_process_started = Signal()
@@ -66,6 +66,7 @@ class IntegrableViewer(QOpenGLWidget):
     signal_animation_started = Signal()
     signal_animation_finished = Signal()
 
+    signal_screen_clicked = Signal(object)
     signal_mode_updated = Signal(str)
     signal_fps_updated = Signal(float)
 
@@ -153,12 +154,6 @@ class IntegrableViewer(QOpenGLWidget):
     def axis(self) -> GLDrawable:
         return self.axis_collection.get_last()
 
-    # DEPRECATED
-    @property
-    def bg(self) -> GLDrawable:
-        self._deprecated_message('bg', 'background')
-        return self.background
-
     @property
     def background(self) -> GLDrawable:
         return self.background_collection.get_last()
@@ -412,78 +407,16 @@ class IntegrableViewer(QOpenGLWidget):
 
         return loaded
 
-    """
-    Load methods by path (DEPRECATED)
-    """
-    @staticmethod
-    def _deprecated_message(old: str, new: str, log: callable = warnings.warn) -> None:
-        log(f'{old} is deprecated. Use {new} instead.', DeprecationWarning, 2)
-
-    def mesh_by_path(self, path: str, *args, **kwargs) -> MeshGL:
-        self._deprecated_message('mesh_by_path()', 'load_mesh()')
-        return self.load_mesh(path, *args, **kwargs)
-
-    def blocks_by_path(self, path: str, *args, **kwargs) -> BlockGL:
-        self._deprecated_message('blocks_by_path()', 'load_blocks()')
-        return self.load_blocks(path, *args, **kwargs)
-
-    def points_by_path(self, path: str, *args, **kwargs) -> PointGL:
-        self._deprecated_message('points_by_path()', 'load_points()')
-        return self.load_points(path, *args, **kwargs)
-
-    def lines_by_path(self, path: str, *args, **kwargs) -> LineGL:
-        self._deprecated_message('lines_by_path()', 'load_lines()')
-        return self.load_lines(path, *args, **kwargs)
-
-    def tubes_by_path(self, path: str, *args, **kwargs) -> TubeGL:
-        self._deprecated_message('tubes_by_path()', 'load_tubes()')
-        return self.load_tubes(path, *args, **kwargs)
-
-    def meshes_by_folder_path(self, path: str, *args, **kwargs) -> list:
-        self._deprecated_message('meshes_by_folder_path()', 'load_mesh_folder()')
-        return self.load_mesh_folder(path, *args, **kwargs)
-
-    def blocks_by_folder_path(self, path: str, *args, **kwargs) -> list:
-        self._deprecated_folder('blocks_by_folder_path()', 'load_blocks_folder()')
-        return self.load_blocks_folder(path, *args, **kwargs)
-
-    def points_by_folder_path(self, path: str, *args, **kwargs) -> list:
-        self._deprecated_folder('points_by_folder_path()', 'load_points_folder()')
-        return self.load_points_folder(path, *args, **kwargs)
-
-    def lines_by_folder_path(self, path: str, *args, **kwargs) -> list:
-        self._deprecated_folder('lines_by_folder_path()', 'load_lines_folder()')
-        return self.load_lines_folder(path, *args, **kwargs)
-
-    def tubes_by_folder_path(self, path: str, *args, **kwargs) -> list:
-        self._deprecated_folder('tubes_by_folder_path()', 'load_tubes_folder()')
-        return self.load_tubes_folder(path, *args, **kwargs)
-
     """
     Export methods
     """
-    def _export_element(self, exporter: callable, path: str, _id: int) -> None:
+    def export_element(self, path: str, _id: int) -> None:
         try:
-            exporter(path, _id)
+            self.model.export(path, _id)
             self.signal_export_success.emit(_id)
         except Exception:
             self.signal_export_failure.emit()
 
-    def export_mesh(self, path: str, _id: int) -> None:
-        self._export_element(self.model.export_mesh, path, _id)
-
-    def export_blocks(self, path: str, _id: int) -> None:
-        self._export_element(self.model.export_blocks, path, _id)
-
-    def export_points(self, path: str, _id: int) -> None:
-        self._export_element(self.model.export_points, path, _id)
-
-    def export_lines(self, path: str, _id: int) -> None:
-        self._export_element(self.model.export_lines, path, _id)
-
-    def export_tubes(self, path: str, _id: int) -> None:
-        self._export_element(self.model.export_tubes, path, _id)
-
     """
     Drawable manipulation
     """
@@ -510,31 +443,48 @@ class IntegrableViewer(QOpenGLWidget):
         self.drawable_collection.delete(_id)
         self.signal_file_modified.emit()
 
-    def recreate(self) -> None:
-        self.axis_collection.recreate()
-        self.background_collection.recreate()
-        self.drawable_collection.recreate()
-        self.update()
+    def update_all(self) -> None:
+        self.makeCurrent()
+        for _id in self.get_all_ids():
+            self.get_drawable(_id).setup_attributes()
+        self.recreate()
 
+    def clear(self) -> None:
+        self.makeCurrent()
+        for _id in self.get_all_ids():
+            self.model.delete(_id)
+            self.drawable_collection.delete(_id)
+        self.signal_file_modified.emit()
+
+    """
+    Drawable retrieval
+    """
     def get_all_ids(self) -> list:
         return self.drawable_collection.get_all_ids()
 
     def get_all_drawables(self) -> list:
         return self.drawable_collection.get_all_drawables()
 
-    def update_all(self) -> None:
-        for _id in self.get_all_ids():
-            self.update_drawable(_id)
+    def get_all_meshes(self) -> list:
+        return self.drawable_collection.filter(MeshGL)
 
-    def clear(self) -> None:
-        for _id in self.get_all_ids():
-            self.delete(_id)
+    def get_all_blocks(self) -> list:
+        return self.drawable_collection.filter(BlockGL)
+
+    def get_all_points(self) -> list:
+        return self.drawable_collection.filter(PointGL)
+
+    def get_all_lines(self) -> list:
+        return self.drawable_collection.filter(LineGL)
+
+    def get_all_tubes(self) -> list:
+        return self.drawable_collection.filter(TubeGL)
 
     """
     Camera handling
     """
     def camera_at(self, _id: int) -> None:
-        self.fit_to_bounds(*self.get_drawable(_id).element.bounding_box)
+        self.fit_to_bounds(*self.get_drawable(_id).bounding_box)
 
     def plan_view(self) -> None:
         self.set_rotation_angle([0.0, 0.0, 0.0])
@@ -584,7 +534,7 @@ class IntegrableViewer(QOpenGLWidget):
         max_all = -np.inf * np.ones(3)
 
         for drawable in drawables:
-            min_bound, max_bound = drawable.element.bounding_box
+            min_bound, max_bound = drawable.bounding_box
             min_all = np.min((min_all, min_bound), axis=0)
             max_all = np.max((max_all, max_bound), axis=0)
 
@@ -654,6 +604,12 @@ class IntegrableViewer(QOpenGLWidget):
             z = self.off_center[2]
             self.proj.ortho(-z, z, -z / aspect, z / aspect, 0.0, 100000.0)
 
+    def recreate(self) -> None:
+        self.axis_collection.recreate()
+        self.background_collection.recreate()
+        self.drawable_collection.recreate()
+        self.update()
+
     """
     Utilities
     """
@@ -675,16 +631,16 @@ class IntegrableViewer(QOpenGLWidget):
             1.0 - (2.0 * y / self.height()),
             1.0])
 
-    def unproject(self, _x, _y, _z, model, view, proj) -> np.ndarray:
+    def unproject(self, x: float, y: float, z: float) -> np.ndarray:
         # Adapted from http://antongerdelan.net/opengl/raycasting.html
-        x, y, z = self.screen_to_ndc(_x, _y, _z)
+        # Note: x, y, z must be normalized to [-1.0, +1.0]
 
         # We'd use `QVector4D(x, y, -1.0, 1.0)`, but PySide2
         # hasn't implemented QMatrix4x4 * QVector4D yet.
         vector = [x, y, -1.0, 1.0]
         temp_matrix = QMatrix4x4(*[e for e in vector for _ in range(4)])
 
-        ray_eye = (proj.inverted()[0] * temp_matrix).column(0)
+        ray_eye = (self.proj.inverted()[0] * temp_matrix).column(0)
         ray_eye = QVector4D(ray_eye.x(), ray_eye.y(), -1.0, 0.0)
 
         # We'd use `ray_eye`, but PySide2
@@ -692,27 +648,29 @@ class IntegrableViewer(QOpenGLWidget):
         vector = [ray_eye.x(), ray_eye.y(), ray_eye.z(), ray_eye.w()]
         temp_matrix = QMatrix4x4(*[e for e in vector for _ in range(4)])
 
-        ray_world = ((view * model).inverted()[0] * temp_matrix).column(0).toVector3D()
+        ray_world = ((self.camera * self.world).inverted()[0] * temp_matrix).column(0).toVector3D()
         ray = ray_world.normalized()
         return np.array([ray.x(), ray.y(), ray.z()])
 
-    def get_origin(self) -> np.ndarray:
-        origin = (self.camera * self.world).inverted()[0].column(3).toVector3D()
-        return np.array([origin.x(), origin.y(), origin.z()])
-
     def ray_from_click(self, x: float, y: float, z: float) -> np.ndarray:
         # Perspective projection is straightforward
-        if self.projection_mode == 'perspective':
-            return self.unproject(x, y, z, self.world, self.camera, self.proj)
+        def perspective_ray() -> np.ndarray:
+            return self.unproject(*self.screen_to_ndc(x, y, z))
 
-        # Orthographic projection "forces" a click in the center
-        return self.unproject(self.width() / 2, self.height() / 2, z,
-                              self.world, self.camera, self.proj)
+        # Orthographic projection "forces" a click in the center (normalized to [-1.0, +1.0])
+        def orthographic_ray() -> np.ndarray:
+            return self.unproject(0.0, 0.0, 0.0)
+
+        if self.projection_mode == 'perspective':
+            return perspective_ray()
+
+        return orthographic_ray()
 
     def origin_from_click(self, x: float, y: float, z: float) -> np.ndarray:
         # Perspective projection is straightforward
-        if self.projection_mode == 'perspective':
-            return self.get_origin()
+        def perspective_origin() -> np.ndarray:
+            origin = (self.camera * self.world).inverted()[0].column(3).toVector3D()
+            return np.array([origin.x(), origin.y(), origin.z()])
 
         # Orthographic projection needs a bit more of vector arithmetic.
         # A click in the center of the screen gives us the perfect ray,
@@ -720,32 +678,42 @@ class IntegrableViewer(QOpenGLWidget):
 
         # But if we don't click in the exact center of screen,
         # we need to trick the origin calculation.
-        ndc = self.screen_to_ndc(x, y, z)
-        aspect = self.width() / self.height()
-        aspect_bias = np.array([1.0, 1.0 / aspect, 0.0])
+        def orthographic_origin() -> np.ndarray:
+            ndc = self.screen_to_ndc(x, y, z)
+            aspect = self.width() / self.height()
+            aspect_bias = np.array([1.0, 1.0 / aspect, 0.0])
 
-        # The idea is to strategically move the camera by an offset, so that
-        # clicking anywhere in the screen will give us the same result as
-        # clicking at the exact center of the screen from a shifted camera.
-        offset = self.off_center[2] * ndc * aspect_bias
+            # The idea is to strategically move the camera by an offset, so that
+            # clicking anywhere in the screen will give us the same result as
+            # clicking at the exact center of the screen from a shifted camera.
+            offset = self.off_center[2] * ndc * aspect_bias
 
-        # Hack to get the origin when the click is not exactly at center of screen.
-        self.camera.translate(*-offset)
-        origin = self.get_origin()
-        self.camera.translate(*offset)
+            # Hack to get the origin when the click is not exactly at center of screen.
+            self.camera.translate(*-offset)
+            origin = perspective_origin()
+            self.camera.translate(*offset)
 
-        return origin
+            return origin
+
+        if self.projection_mode == 'perspective':
+            return perspective_origin()
+
+        return orthographic_origin()
 
     def get_normal(self, origin_list, ray_list: list) -> np.ndarray:
-        if self.projection_mode == 'perspective':
-            # Perspective: Same origins, different rays
-            normal = np.cross(*ray_list)
-        else:
+        # Perspective: Same origins, different rays
+        def perspective_normal() -> np.ndarray:
+            return utils.normalize(np.cross(*ray_list))
+
+        def orthographic_normal() -> np.ndarray:
             # Orthographic: Same rays, different origins
             origin_diff = np.diff(origin_list, axis=0)[0]
-            normal = np.cross(ray_list[0], origin_diff)
+            return utils.normalize(np.cross(ray_list[0], origin_diff))
 
-        return normal / np.linalg.norm(normal)
+        if self.projection_mode == 'perspective':
+            return perspective_normal()
+
+        return orthographic_normal()
 
     @staticmethod
     def angles_from_vectors(normal: np.ndarray, up: np.ndarray) -> np.ndarray:
@@ -765,8 +733,7 @@ class IntegrableViewer(QOpenGLWidget):
         # In perspective projection, the origin is the same.
         origin = origin_list[0]
         normal = self.get_normal(origin_list, ray_list)
-        up = np.cross(ray_list[0], normal)
-        up /= np.linalg.norm(up)
+        up = utils.normalize(np.cross(ray_list[0], normal))
 
         # Emit description of the slice
         self.signal_slice_description.emit({
@@ -777,38 +744,47 @@ class IntegrableViewer(QOpenGLWidget):
 
     def slice_meshes(self, origin: np.ndarray, normal: np.ndarray, include_hidden: bool = False) -> list:
         # By default, slice only visible meshes
-        meshes = [m.element for m in self.drawable_collection.filter(MeshGL)
-                  if m.is_visible or include_hidden]
+        meshes = list(filter(lambda m: m.is_visible or include_hidden, self.get_all_meshes()))
 
         return self.model.slice_meshes(origin, normal, meshes)
 
     def slice_blocks(self, origin: np.ndarray, normal: np.ndarray, include_hidden: bool = True) -> list:
         # By default, slice visible and hidden blocks
-        blocks = [m.element for m in self.drawable_collection.filter(BlockGL)
-                  if m.is_visible or include_hidden]
+        blocks = list(filter(lambda m: m.is_visible or include_hidden, self.get_all_blocks()))
 
         return self.model.slice_blocks(origin, normal, blocks)
 
+    def intersect_meshes(self, origin: np.ndarray, ray: np.ndarray, include_hidden: bool = False) -> list:
+        # By default, intersect only visible meshes
+        meshes = list(filter(lambda m: m.is_visible or include_hidden, self.get_all_meshes()))
+
+        return self.model.intersect_meshes(origin, ray, meshes)
+
+    def intersect_lines(self, origin: np.ndarray, ray: np.ndarray, include_hidden: bool = False) -> list:
+        # By default, intersect only visible lines
+        lines = list(filter(lambda m: m.is_visible or include_hidden, self.get_all_lines()))
+
+        return self.model.intersect_lines(origin, ray, lines)
+
+    def intersect_elements(self, origin: np.ndarray, ray: np.ndarray, include_hidden: bool = False) -> list:
+        results = []
+        results += self.intersect_meshes(origin, ray, include_hidden)
+        results += self.intersect_lines(origin, ray, include_hidden)
+
+        self.signal_elements_detected.emit(results)
+
     def measure_from_rays(self, origin_list: list, ray_list: list) -> None:
-        meshes = [m.element for m in self.drawable_collection.filter(MeshGL) if m.is_visible]
+        meshes = list(filter(lambda m: m.is_visible, self.get_all_meshes()))
 
         results = self.model.measure_from_rays(origin_list, ray_list, meshes)
         self.signal_mesh_distances.emit(results)
 
-    def detect_mesh_intersection(self, x: float, y: float, z: float) -> None:
-        ray = self.ray_from_click(x, y, z)
-        origin = self.origin_from_click(x, y, z)
-        meshes = [m.element for m in self.drawable_collection.filter(MeshGL) if m.is_visible]
-
-        results = self.model.detect_mesh_intersection(ray, origin, meshes)
-        self.signal_mesh_clicked.emit(results)
-
     def cross_section(self, origin: np.ndarray, normal: np.ndarray) -> None:
         self.drawable_collection.update_uniform('CrossSection', 'plane_origin', *origin)
         self.drawable_collection.update_uniform('CrossSection', 'plane_normal', *normal)
 
     def set_cross_section(self, status: bool) -> None:
-        for m in self.drawable_collection.filter(MeshGL):
+        for m in self.get_all_meshes():
             m.is_cross_sectionable = status
 
     """
